==> billogram-foundation-3.5.0.tgz: package/modules/components/dropdown/Dropdown.js <==
import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import onClickOutside from 'react-onclickoutside';
import invariant from 'invariant';

import DropdownActionOption from './option-action/DropdownActionOption';
import DropdownLinkOption from './option-link/DropdownLinkOption';
import DropdownRouteOption from './option-route/DropdownRouteOption';
import DropdownValueOption from './option-value/DropdownValueOption';

import { dropdownOptionsShape } from './utils/propTypes';
import getScrollTopHighlightedIndexChanged from './utils/getScrollTopHighlightedIndexChanged';
import getScrollTopExpanded from './utils/getScrollTopExpanded';

import isNullOrEmpty from '../../isNullOrEmpty';
import modulus from '../../modulus';


export default ({
    block,
    displayName,
    uiPropDefaultType = PropTypes.string,
    uiPropDefaultValue = '',
    uiPropGetValue = (ui) => ui
} = {}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    const ActionOption = DropdownActionOption({ block, displayName: `${displayName}ActionOption` });
    const LinkOption = DropdownLinkOption({ block, displayName: `${displayName}LinkOption` });
    const RouteOption = DropdownRouteOption({ block, displayName: `${displayName}RouteOption` });
    const ValueOption = DropdownValueOption({ block, displayName: `${displayName}ValueOption` });

    return onClickOutside(React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        contextTypes: {
            router: PropTypes.object
        },

        propTypes: {
            disabled: PropTypes.bool,
            emptyText: PropTypes.string,
            expanded: PropTypes.bool,
            focus: PropTypes.bool,
            highlightedIndex: PropTypes.number,
            onChange: PropTypes.func,
            onCollapse: PropTypes.func,
            onExpand: PropTypes.func,
            onHighlightedIndexChange: PropTypes.func,
            options: dropdownOptionsShape,
            scrollable: PropTypes.bool,
            showLinkRouteOptionArrow: PropTypes.bool,
            ui: uiPropDefaultType,
            value: PropTypes.any,
            width: PropTypes.oneOfType([
                PropTypes.number,
                PropTypes.oneOf(['auto'])
            ])
        },

        getDefaultProps() {
            return {
                disabled: false,
                emptyText: '',
                expanded: false,
                focus: false,
                highlightedIndex: -1,
                onChange: (/* value */) => {},
                onCollapse: () => {},
                onExpand: () => {},
                onHighlightedIndexChange: (/* highlightedIndex */) => {},
                options: [],
                scrollable: false,
                showLinkRouteOptionArrow: true,
                ui: uiPropDefaultValue,
                value: '',
                width: null
            };
        },

        componentWillMount() {
            this.components = [];
            this.token = null;
            this.typed = [];
        },

        componentDidMount() {
            if (this.props.focus) {
                ReactDOM.findDOMNode(this.component).focus();
            }
        },

        componentDidUpdate(prevProps) {
            if (!prevProps.expanded && this.props.expanded) {
                this.components.options.scrollTop = getScrollTopExpanded(this.props.value, this.props.options, this.components, (index) => {
                    return ReactDOM.findDOMNode(this.components[index]).offsetHeight;
                });
            } else if (this.props.expanded && prevProps.highlightedIndex !== this.props.highlightedIndex) {
                this.components.options.scrollTop = getScrollTopHighlightedIndexChanged(
                    prevProps.highlightedIndex,
                    this.props.highlightedIndex,
                    this.typed.length >= 0,
                    this.props.options,
                    this.components,
                    (index) => {
                        return ReactDOM.findDOMNode(this.components[index]).offsetHeight;
                    }
                );
            }
        },

        handleClickOutside() {
            if (this.props.expanded) {
                this.collapse();
            }
        },

        onKeyDown(evt) {
            switch (evt.which) {
                case 9: // tab
                    if (this.props.expanded) {
                        // don't tab to next form control when expanded.
                        evt.stopPropagation();
                        evt.preventDefault();
                    }

                    break;

                case 13: // enter
                case 32: // space
                    // don't scroll on page or risk submitting the form on enter here.
                    evt.stopPropagation();
                    evt.preventDefault();

                    if (this.props.expanded) {
                        const option = this.props.options[this.props.highlightedIndex];
                        if (option) {
                            if (option.type === 'value') {
                                this.change(option.value);
                            } else {
                                const component = this.components[this.props.highlightedIndex];
                                if (component) {
                                    ReactDOM.findDOMNode(component).click();
                                }
                            }
                        }

                        this.collapse();
                    } else {
                        this.expand();
                    }

                    break;

                case 27: // escape
                    if (this.props.expanded) {
                        this.collapse();
                    }

                    break;

                case 38: // up arrow
                    if (this.props.expanded) {
                        // don't scroll on page.
                        evt.stopPropagation();
                        evt.preventDefault();

                        this.changeHighlightedIndexByArrawKey(-1);
                    }

                    break;

                case 40: // down arrow
                    // don't scroll on page.
                    evt.stopPropagation();
                    evt.preventDefault();

                    if (this.props.expanded) {
                        this.changeHighlightedIndexByArrawKey(1);
                    } else {
                        this.expand();
                    }

                    break;

                default:
                    break;
            }
        },

        onKeyPress(evt) {
            clearTimeout(this.token);
            this.token = setTimeout(() => {
                this.typed = [];
                this.token = null;
            }, 250);

            this.typed.push(evt.which);
            this.find(String.fromCharCode(...this.typed));
        },

        onToggle() {
            if (this.props.expanded) {
                this.collapse();
            } else {
                this.expand();
            }
        },

        change(value) {
            if (this.props.value === value) {
                this.props.onCollapse();
            } else {
                this.props.onChange(value);
            }
        },

        changeHighlightedIndexByArrawKey(increment) {
            if (this.props.options.length === 0) {
                return;
            }

            if (this.props.options.every((option) => option.disabled)) {
                return;
            }

            const highlightedIndex = this.props.highlightedIndex > -1
                ? this.props.highlightedIndex
                : increment > 0 ? -1 : this.props.options.length;

            const index = this.getNextHighlightedIndex(highlightedIndex, increment);

            if (this.props.highlightedIndex === index) {
                return;
            }

            this.props.onHighlightedIndexChange(index);
        },

        collapse() {
            this.props.onCollapse();
        },

        expand() {
            this.props.onExpand();
        },

        find(text) {
            const re = new RegExp('^' + text, 'i');

            const options = this.props.options.filter((option) => re.test(option.text));
            if (options.length === 0) {
                return;
            }

            const index = this.props.options.indexOf(options[0]);
            if (this.props.expanded) {
                this.props.onHighlightedIndexChange(index);
            }
        },

        getNextHighlightedIndex(index, increment) {
            const newIndex = modulus(index + increment, this.props.options.length);
            const option = this.props.options[newIndex];

            return option.disabled
                ? this.getNextHighlightedIndex(newIndex, increment)
                : newIndex;
        },

        renderActionOption(option, index) {
            const { disabled, onClick, text, value } = option;

            const highlighted = this.props.highlightedIndex === index;

            return (
                <ActionOption
                    text={ text }
                    disabled={ disabled }
                    highlighted={ highlighted }
                    key={ index }
                    onClick={ onClick }
                    ref={ (c) => this.components[index] = c }
                    value={ value } />
            );
        },

        renderLinkOption(option, index) {
            const { showLinkRouteOptionArrow } = this.props;
            const { disabled, text, to } = option;

            const highlighted = this.props.highlightedIndex === index;

            return (
                <LinkOption
                    text={ text }
                    disabled={ disabled }
                    highlighted={ highlighted }
                    key={ index }
                    showArrow={ showLinkRouteOptionArrow }
                    ref={ (c) => this.components[index] = c }
                    to={ to } />
            );
        },

        renderRouteOption(option, index) {
            const { showLinkRouteOptionArrow } = this.props;
            const { disabled, text, to, value } = option;

            const highlighted = this.props.highlightedIndex === index;
            const selected = !isNullOrEmpty(value) && this.props.value === value;

            return (
                <RouteOption
                    text={ text }
                    disabled={ disabled }
                    highlighted={ highlighted }
                    key={ index }
                    ref={ (c) => this.components[index] = c }
                    selected={ selected }
                    showArrow={ showLinkRouteOptionArrow }
                    to={ to } />
            );
        },

        renderValueOption(option, index) {
            const { disabled, text, value } = option;

            const highlighted = this.props.highlightedIndex === index;
            const selected = !isNullOrEmpty(value) && this.props.value === value;

            return (
                <ValueOption
                    text={ text }
                    disabled={ disabled }
                    highlighted={ highlighted }
                    key={ value }
                    onChange={ this.change }
                    ref={ (c) => this.components[index] = c }
                    selected={ selected }
                    value={ value } />
            );
        },

        renderOption(option, index) {
            switch (option.type) {
                case 'action':
                    return this.renderActionOption(option, index);

                case 'link':
                    return this.renderLinkOption(option, index);

                case 'route':
                    return this.renderRouteOption(option, index);

                case 'value':
                    return this.renderValueOption(option, index);

                default:
                    throw Error(`Unknown dropdown option type '${option.type}'`);
            }
        },

        renderOptions() {
            if (!this.props.expanded) {
                return null;
            }

            return (
                <div className={ block('options') } ref={ (c) => this.components.options = c }>
                    { [].map.call(this.props.options, this.renderOption) }
                </div>
            );
        },

        renderContent() {
            const { disabled, emptyText, options, value } = this.props;

            const foptions = options.filter((option) => !isNullOrEmpty(option.value) && option.value === value);
            const text = foptions.length > 0
                ? !isNullOrEmpty(foptions[0].textSelected) ? foptions[0].textSelected : foptions[0].text
                : emptyText;

            return (
                <div
                    className={ block('content') }
                    tabIndex={ disabled ? null : '0' }>
                    <span className={ block('content-text') }>{ text }</span>
                    <i className={ block('content-toggle-icon') } />
                </div>
            );
        },

        render() {
            const { disabled, expanded, options, scrollable, ui, width } = this.props;
            const activeRoute = options
                .filter((option) => option.type === 'route')
                .some((option) => this.context.router.isActive(option.to, true));

            return (
                <div
                    className={ block({ activeRoute, collapsed: !expanded, disabled, expanded, scrollable }, uiPropGetValue(ui)) }
                    onClick={ this.onToggle }
                    onKeyDown={ this.onKeyDown }
                    onKeyPress={ this.onKeyPress }
                    ref={ (c) => this.component = c }
                    style={ { width } }>
                    { this.renderContent() }
                    { this.renderOptions() }
                </div>
            );
        }

    }));
};

==> billogram-foundation-3.11.0.tgz: package/modules/components/dropdown/Dropdown.js <==
import React, { PropTypes } from 'react';
import ReactDOM from 'react-dom';
import PureRenderMixin from 'react-addons-pure-render-mixin';
import onClickOutside from 'react-onclickoutside';
import invariant from 'invariant';

import DropdownActionOption from './option-action/DropdownActionOption';
import DropdownLinkOption from './option-link/DropdownLinkOption';
import DropdownRouteOption from './option-route/DropdownRouteOption';
import DropdownValueOption from './option-value/DropdownValueOption';

import { dropdownOptionsShape } from './utils/propTypes';
import getScrollTopHighlightedIndexChanged from './utils/getScrollTopHighlightedIndexChanged';
import getScrollTopExpanded from './utils/getScrollTopExpanded';

import isNullOrEmpty from '../../utils/isNullOrEmpty';
import modulus from '../../utils/modulus';


export default ({
    block,
    displayName,
    uiPropDefaultType = PropTypes.string,
    uiPropDefaultValue = '',
    uiPropGetValue = (ui) => ui
} = {}) => {

    invariant(block !== undefined, 'Missing purebem block.');
    invariant(!isNullOrEmpty(displayName), 'Missing display name.');

    const ActionOption = DropdownActionOption({ block, displayName: `${displayName}ActionOption` });
    const LinkOption = DropdownLinkOption({ block, displayName: `${displayName}LinkOption` });
    const RouteOption = DropdownRouteOption({ block, displayName: `${displayName}RouteOption` });
    const ValueOption = DropdownValueOption({ block, displayName: `${displayName}ValueOption` });

    return onClickOutside(React.createClass({

        displayName,

        mixins: [PureRenderMixin],

        contextTypes: {
            router: PropTypes.object
        },

        propTypes: {
            disabled: PropTypes.bool,
            emptyText: PropTypes.string,
            expanded: PropTypes.bool,
            focus: PropTypes.bool,
            highlightedIndex: PropTypes.number,
            onChange: PropTypes.func,
            onCollapse: PropTypes.func,
            onExpand: PropTypes.func,
            onHighlightedIndexChange: PropTypes.func,
            options: dropdownOptionsShape,
            scrollable: PropTypes.bool,
            showLinkRouteOptionArrow: PropTypes.bool,
            ui: uiPropDefaultType,
            value: PropTypes.any,
            width: PropTypes.oneOfType([
                PropTypes.number,
                PropTypes.oneOf(['auto'])
            ])
        },

        getDefaultProps() {
            return {
                disabled: false,
                emptyText: '',
                expanded: false,
                focus: false,
                highlightedIndex: -1,
                onChange: (/* value */) => {},
                onCollapse: () => {},
                onExpand: () => {},
                onHighlightedIndexChange: (/* highlightedIndex */) => {},
                options: [],
                scrollable: false,
                showLinkRouteOptionArrow: true,
                ui: uiPropDefaultValue,
                value: '',
                width: null
            };
        },

        componentWillMount() {
            this.components = [];
            this.token = null;
            this.typed = [];
        },

        componentDidMount() {
            if (this.props.focus) {
                ReactDOM.findDOMNode(this.component).focus();
            }
        },

        componentDidUpdate(prevProps) {
            if (!prevProps.expanded && this.props.expanded) {
                this.components.options.scrollTop = getScrollTopExpanded(this.props.value, this.props.options, this.components, (index) => {
                    return ReactDOM.findDOMNode(this.components[index]).offsetHeight;
                });
            } else if (this.props.expanded && prevProps.highlightedIndex !== this.props.highlightedIndex) {
                this.components.options.scrollTop = getScrollTopHighlightedIndexChanged(
                    prevProps.highlightedIndex,
                    this.props.highlightedIndex,
                    this.typed.length >= 0,
                    this.props.options,
                    this.components,
                    (index) => {
                        return ReactDOM.findDOMNode(this.components[index]).offsetHeight;
                    }
                );
            }
        },

        handleClickOutside() {
            if (this.props.expanded) {
                this.collapse();
            }
        },

        onKeyDown(evt) {
            switch (evt.which) {
                case 9: // tab
                    if (this.props.expanded) {
                        // don't tab to next form control when expanded.
                        evt.stopPropagation();
                        evt.preventDefault();
                    }

                    break;

                case 13: // enter
                case 32: // space
                    // don't scroll on page or risk submitting the form on enter here.
                    evt.stopPropagation();
                    evt.preventDefault();

                    if (this.props.expanded) {
                        const option = this.props.options[this.props.highlightedIndex];
                        if (option) {
                            if (option.type === 'value') {
                                this.change(option.value);
                            } else {
                                const component = this.components[this.props.highlightedIndex];
                                if (component) {
                                    ReactDOM.findDOMNode(component).click();
                                }
                            }
                        }

                        this.collapse();
                    } else {
                        this.expand();
                    }

                    break;

                case 27: // escape
                    if (this.props.expanded) {
                        this.collapse();
                    }

                    break;

                case 38: // up arrow
                    if (this.props.expanded) {
                        // don't scroll on page.
                        evt.stopPropagation();
                        evt.preventDefault();

                        this.changeHighlightedIndexByArrawKey(-1);
                    }

                    break;

                case 40: // down arrow
                    // don't scroll on page.
                    evt.stopPropagation();
                    evt.preventDefault();

                    if (this.props.expanded) {
                        this.changeHighlightedIndexByArrawKey(1);
                    } else {
                        this.expand();
                    }

                    break;

                default:
                    break;
            }
        },

        onKeyPress(evt) {
            clearTimeout(this.token);
            this.token = setTimeout(() => {
                this.typed = [];
                this.token = null;
            }, 250);

            this.typed.push(evt.which);
            this.find(String.fromCharCode(...this.typed));
        },

        onToggle() {
            if (this.props.expanded) {
                this.collapse();
            } else {
                this.expand();
            }
        },

        change(value) {
            if (this.props.value === value) {
                this.props.onCollapse();
            } else {
                this.props.onChange(value);
            }
        },

        changeHighlightedIndexByArrawKey(increment) {
            if (this.props.options.length === 0) {
                return;
            }

            if (this.props.options.every((option) => option.disabled)) {
                return;
            }

            const highlightedIndex = this.props.highlightedIndex > -1
                ? this.props.highlightedIndex
                : increment > 0 ? -1 : this.props.options.length;

            const index = this.getNextHighlightedIndex(highlightedIndex, increment);

            if (this.props.highlightedIndex === index) {
                return;
            }

            this.props.onHighlightedIndexChange(index);
        },

        collapse() {
            this.props.onCollapse();
        },

        expand() {
            this.props.onExpand();
        },

        find(text) {
            const re = new RegExp('^' + text, 'i');

            const options = this.props.options.filter((option) => re.test(option.text));
            if (options.length === 0) {
                return;
            }

            const index = this.props.options.indexOf(options[0]);
            if (this.props.expanded) {
                this.props.onHighlightedIndexChange(index);
            }
        },

        getNextHighlightedIndex(index, increment) {
            const newIndex = modulus(index + increment, this.props.options.length);
            const option = this.props.options[newIndex];

            return option.disabled
                ? this.getNextHighlightedIndex(newIndex, increment)
                : newIndex;
        },

        renderActionOption(option, index) {
            const { disabled, onClick, text, value } = option;

            const highlighted = this.props.highlightedIndex === index;

            return (
                <ActionOption
                    text={ text }
                    disabled={ disabled }
                    highlighted={ highlighted }
                    key={ index }
                    onClick={ onClick }
                    ref={ (c) => this.components[index] = c }
                    value={ value } />
            );
        },

        renderLinkOption(option, index) {
            const { showLinkRouteOptionArrow } = this.props;
            const { disabled, text, to } = option;

            const highlighted = this.props.highlightedIndex === index;

            return (
                <LinkOption
                    text={ text }
                    disabled={ disabled }
                    highlighted={ highlighted }
                    key={ index }
                    showArrow={ showLinkRouteOptionArrow }
                    ref={ (c) => this.components[index] = c }
                    to={ to } />
            );
        },

        renderRouteOption(option, index) {
            const { showLinkRouteOptionArrow } = this.props;
            const { disabled, text, to, value } = option;

            const highlighted = this.props.highlightedIndex === index;
            const selected = !isNullOrEmpty(value) && this.props.value === value;

            return (
                <RouteOption
                    text={ text }
                    disabled={ disabled }
                    highlighted={ highlighted }
                    key={ index }
                    ref={ (c) => this.components[index] = c }
                    selected={ selected }
                    showArrow={ showLinkRouteOptionArrow }
                    to={ to } />
            );
        },

        renderValueOption(option, index) {
            const { disabled, text, value } = option;

            const highlighted = this.props.highlightedIndex === index;
            const selected = !isNullOrEmpty(value) && this.props.value === value;

            return (
                <ValueOption
                    text={ text }
                    disabled={ disabled }
                    highlighted={ highlighted }
                    key={ value }
                    onChange={ this.change }
                    ref={ (c) => this.components[index] = c }
                    selected={ selected }
                    value={ value } />
            );
        },

        renderOption(option, index) {
            switch (option.type) {
                case 'action':
                    return this.renderActionOption(option, index);

                case 'link':
                    return this.renderLinkOption(option, index);

                case 'route':
                    return this.renderRouteOption(option, index);

                case 'value':
                    return this.renderValueOption(option, index);

                default:
                    throw Error(`Unknown dropdown option type '${option.type}'`);
            }
        },

        renderOptions() {
            if (!this.props.expanded) {
                return null;
            }

            return (
                <div className={ block('options') } ref={ (c) => this.components.options = c }>
                    { [].map.call(this.props.options, this.renderOption) }
                </div>
            );
        },

        renderContent() {
            const { disabled, emptyText, options, value } = this.props;

            const foptions = options.filter((option) => !isNullOrEmpty(option.value) && option.value === value);
            const text = foptions.length > 0
                ? !isNullOrEmpty(foptions[0].textSelected) ? foptions[0].textSelected : foptions[0].text
                : emptyText;

            return (
                <div
                    className={ block('content') }
                    tabIndex={ disabled ? null : '0' }>
                    <span className={ block('content-text') }>{ text }</span>
                    <i className={ block('content-toggle-icon') } />
                </div>
            );
        },

        render() {
            const { disabled, expanded, options, scrollable, ui, width } = this.props;
            const activeRoute = options
                .filter((option) => option.type === 'route')
                .some((option) => this.context.router.isActive(option.to, true));

            return (
                <div
                    className={ block({ activeRoute, collapsed: !expanded, disabled, expanded, scrollable }, uiPropGetValue(ui)) }
                    onClick={ this.onToggle }
                    onKeyDown={ this.onKeyDown }
                    onKeyPress={ this.onKeyPress }
                    ref={ (c) => this.component = c }
                    style={ { width } }>
                    { this.renderContent() }
                    { this.renderOptions() }
                </div>
            );
        }

    }));
};
